DEVICE_REGISTER = "api/mobile/register/new"
STUDENT_LIST = "api/mobile/register/hebe"

DATA_ROOT = "api/mobile"
DATA_BY_PUPIL = "byPupil"
DATA_BY_PERSON = "byPerson"
DATA_BY_PERIOD = "byPeriod"
DATA_DELETED = "deleted"

DATA_ADDRESSBOOK = "addressbook"
DATA_INTERNAL_TIME = "internal/time"
DATA_LUCKY_NUMBER = "school/lucky"

DATA_EXAM = "exam"
DATA_ATTENDANCE = "lesson"
DATA_GRADE = "grade"
DATA_GRADE_SUMMARY = "grade/summary"
DATA_GRADE_AVERAGE = "grade/average"
DATA_HOMEWORK = "homework"
DATA_MESSAGE = "message"
DATA_TIMETABLE = "schedule"
DATA_TIMETABLE_CHANGES = "schedule/changes"
